package maincontroller;

import gui.AddressBookModel;
import gui.GUIinterface;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.crypto.Cipher;
import javax.crypto.CipherInputStream;
import javax.crypto.CipherOutputStream;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.DESKeySpec;
import javax.swing.JOptionPane;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import completemodel.CompletemodelFactory;
import completemodel.CompletemodelPackage;
import completemodel.Contact;
import completemodel.ContactCollection;

public class MainController {
	
	private ContactCollection contactCollection;
	private GUIinterface guiInterface;
	private AddressBookModel addressBookModel;
	private String contactFile = "contact.xml";
	private List <Contact> sortedContact;
	private List <Contact> searchedContact;
	

	
	public MainController() {
	
		contactCollection = load();
		addressBookModel = new AddressBookModel(contactCollection.getContacts());
		
	}
	
		
	private ContactCollection load() {
				
		// decryption 
		decrypt();
		
		//loading the resource set
		ResourceSet resSet = getRecourceSet();
		ContactCollection contactCollection = null;
		try {
			Resource resource = resSet.getResource(URI
				.createURI(contactFile), true);
			contactCollection = (ContactCollection) resource.getContents().get(0);
		} catch (Exception e) {
			//if not exists create a contact collection
			contactCollection = CompletemodelFactory.eINSTANCE.createContactCollection();
		}
			
		// encrypt
		encrypt();
		//clears the contactFile contents
		clearFile();
		return contactCollection;
	}



	//resource set
	private ResourceSet getRecourceSet() {
		
		// Initializing model
		CompletemodelPackage.eINSTANCE.eClass();
		
		// in order to register the user.xml file
		Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
		Map<String, Object> m = reg.getExtensionToFactoryMap();
		//XMIResourceFactoryImpl
		m.put("xml", new XMIResourceFactoryImpl());

		// get the resource set
		return new ResourceSetImpl();
	} 
	
	
	// save
	public void save() throws IOException{
		
		// decrypt
		decrypt();
		
		ResourceSet resSet = getRecourceSet();
		// resource creation
		Resource resource = resSet.createResource(URI
				.createURI(contactFile));

		resource.getContents().add(contactCollection);
			resource.save(Collections.EMPTY_MAP);
			
		//encrypt
		encrypt();
		//clears the contactFile contents
		clearFile();
			
	}
	
	// creates contact 
	public void createContact(String name, String lastName, String sex,String phone, String mobile, 
			String fax,String mail, String altMail,String country,String city,String zipCode,
			String street,String apartment,String additionalInfo) {
		Contact contact = CompletemodelFactory.eINSTANCE.createContact();
		contact.setName(name);
		contact.setLastname(lastName);
		contact.setSex(sex);
		contact.setPhone(phone);
		contact.setMobile(mobile);
		contact.setFax(fax);
		contact.setMail(mail);
		contact.setaltEmail(altMail);
		contact.setCounrtry(country);
		contact.setCity(city);
		contact.setStreet(street);
		contact.setZipcode(zipCode);
		contact.setApartment(apartment);
		contact.setAdditional_info(additionalInfo);
		contactCollection.getContacts().add(contact);
		int index = contactCollection.getContacts().indexOf(contact);
		addressBookModel.contactAdded(index);
	
	}
	
	// edit contact ....
    public void editContact(int index, String name, String lastName, String sex,String phone, String mobile,
                    String fax,String mail, String altMail,String country,String city,String zipCode,String street,String apartment,String additionalInfo) throws IOException{
                    if(index !=-1){
                            System.out.println(name + lastName  + country + sex + city + zipCode + street + apartment + mobile + phone + fax + altMail + additionalInfo + additionalInfo);
                            Contact contact = addressBookModel.getElementAt(index);
                            contact.setName(name);
                            contact.setLastname(lastName);
                            contact.setSex(sex);
                            contact.setPhone(phone);
                            contact.setMobile(mobile);
                            contact.setFax(fax);
                            contact.setMail(mail);
                            contact.setaltEmail(altMail);
                            contact.setCounrtry(country);
                            contact.setCity(city);
                            contact.setZipcode(zipCode);
                            contact.setStreet(street);
                            contact.setApartment(apartment);
                            contact.setAdditional_info(additionalInfo);
                            addressBookModel.contactEdited(index);
                            save();
                    }

    }
	
	//search contact
	public List searchContactLastName(String lastName){
		searchedContact = new ArrayList<Contact>();
		
		for (int i=0;i <addressBookModel.getSize(); i++){
			if (addressBookModel.getElementAt(i).getLastname().equalsIgnoreCase(lastName)){
			    searchedContact.add( addressBookModel.getElementAt(i) );
			}
		} 
		return searchedContact;
	}
	
	//public boolean duplicateFinder(String email){
		
		
		
	//}
	// sorts contact
	@SuppressWarnings("rawtypes")
	public List sortContact(){
		 sortedContact = new ArrayList<Contact>();
		for(int j=0; j<addressBookModel.getSize();j++){
			sortedContact.add(addressBookModel.getElementAt(j));
		}
		return sortedContact; 
	}
	
	
	// creates user ....
	
	public void createUser(String name, String lastName, String mobile, String email){
		Contact contact = CompletemodelFactory.eINSTANCE.createContact();
		contact.setName(name);
		contact.setLastname(lastName);
		contact.setMobile(mobile);
		contact.setMail(email);
		contactCollection.getContacts().add(contact);
		int index = contactCollection.getContacts().indexOf(contact);
		addressBookModel.contactAdded(index);
	}
	
	// opens the requested window 
	public void open(GUIinterface iface){
		guiInterface = iface;
		iface.open();
	}
	
	
	
	public void encrypt(){
		try {
			String key = getHashedFromFile();
			FileInputStream fis;
			fis = new FileInputStream(contactFile);
			FileOutputStream fos = new FileOutputStream("cipher.xml");
			encrypt(key, fis, fos);
		} catch (FileNotFoundException e1) {
			System.out.println("file not found exception  " + e1);
		} catch (Throwable ed) {
			System.out.println("throwable exception   " + ed);
		}
	}
	
	
	public void encrypt(String k){
		try {
			String key = k;
			FileInputStream fis;
			fis = new FileInputStream(contactFile);
			FileOutputStream fos = new FileOutputStream("cipher.xml");
			encrypt(key, fis, fos);
		} catch (FileNotFoundException e1) {
			System.out.println("file not found exception  " + e1);
		} catch (Throwable ed) {
			System.out.println("throwable exception   " + ed);
		}
	}
	
	public void decrypt(){
		// decrypt
		FileInputStream fis2;
		FileOutputStream fos2;
		File f = new File(contactFile);
		// if the file is already decrypt
		 try {
			 if(f.length() != 0){
			// checks if the password is correct 
			String key = getHashedFromFile();
		    fis2 = new FileInputStream("cipher.xml");
		    fos2 = new FileOutputStream(contactFile);
		    decrypt(key, fis2, fos2);
		}
		    // if the password is wrong
			else{
		       	JOptionPane.showMessageDialog( null," the file size is zero !!!" );
			}
		} catch (Throwable e) { e.printStackTrace(); }
	}
	
	public void decrypt(String k){
		// decrypt
		FileInputStream fis2;
		FileOutputStream fos2;
		File f = new File(contactFile);
		// if the file is already decrypt
		 try {
			 if(f.length() != 0){
			// checks if the password is correct 
			String key = k;
		    fis2 = new FileInputStream("cipher.xml");
		    fos2 = new FileOutputStream(contactFile);
		    decrypt(key, fis2, fos2);
		}
		    // if the password is wrong
			else{
		       	JOptionPane.showMessageDialog( null," the file size is zero !!!" );
			}
		} catch (Throwable e) { e.printStackTrace(); }
	}
	
	
	public void encrypt(String key, InputStream is, OutputStream os) throws Throwable {
		encryptOrDecrypt(key, Cipher.ENCRYPT_MODE, is, os);
	}
	
	public void decrypt(String key, InputStream is, OutputStream os) throws Throwable {
		encryptOrDecrypt(key, Cipher.DECRYPT_MODE, is, os);
	}

	public void encryptOrDecrypt(String key, int mode, InputStream is, OutputStream os) throws Throwable {

		DESKeySpec dks = new DESKeySpec(key.getBytes());
		SecretKeyFactory skf = SecretKeyFactory.getInstance("DES");
		SecretKey desKey = skf.generateSecret(dks);
		Cipher cipher = Cipher.getInstance("DES"); // DES/ECB/PKCS5Padding for SunJCE

		if (mode == Cipher.ENCRYPT_MODE) {
			cipher.init(Cipher.ENCRYPT_MODE, desKey);
			CipherInputStream cis = new CipherInputStream(is, cipher);
			doCopy(cis, os);
		} else if (mode == Cipher.DECRYPT_MODE) {
			cipher.init(Cipher.DECRYPT_MODE, desKey);
			CipherOutputStream cos = new CipherOutputStream(os, cipher);
			doCopy(is, cos);
		}
	}

	public void doCopy(InputStream is, OutputStream os) throws IOException {
		byte[] bytes = new byte[64];
		int numBytes;
		while ((numBytes = is.read(bytes)) != -1) {
			os.write(bytes, 0, numBytes);
		}
		os.flush();
		os.close();
		is.close();
	}

	
	// clears content of contact.xml file after encrypting to other file.
	public void clearFile(){
		PrintWriter wr;
		try {
			wr = new PrintWriter(new FileWriter(new File(contactFile)));
			wr.println("no contact avilable ");
			wr.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	// hashing the password
	
	public String getMd5(String string) throws UnsupportedEncodingException, NoSuchAlgorithmException {
        MessageDigest md = MessageDigest.getInstance("MD5");
        md.update(string.getBytes("UTF-8"));
        return getMd5(md);
    }


	private  String getMd5(MessageDigest md) {
        String hashtext = "";
        byte[] digest = md.digest();
        BigInteger bigInt = new BigInteger(1,digest);
        hashtext = bigInt.toString(16);
        // Now we need to zero pad it if you actually want the full 32 chars.
        while(hashtext.length() < 32 )
            hashtext = "0"+hashtext;
        return hashtext;
    }
	
	public String getHashedFromFile() throws IOException{
		BufferedReader br = new BufferedReader(new FileReader(new File("user.xml")));
    	String hashedValue= br.readLine();
    	br.close();
    	return hashedValue;
	}
	
	
	public int getAddressBookSize() {
		//size of address book
		return addressBookModel.getSize();
	}


	public void deleteContact(String mail) throws IOException{
		// TODO Auto-generated method stub
		for (int i=0; i < addressBookModel.getSize(); i ++ ){
			if (addressBookModel.getElementAt(i).getMail().equalsIgnoreCase(mail)){
				addressBookModel.contactDeleted(i);
				contactCollection.getContacts().remove(i);
				save();
			}
			
		}
			
	}


	
}